/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Declarations for a BIBTEX entry data type.
 */


#ifndef BIBTEX_ENTRY_H
#define BIBTEX_ENTRY_H


#include <config.h>

#include <stdlib.h>
#include <inttypes.h>


/** @brief Field in a BIBTEX entry. */
typedef struct
{
    /** @brief Field name. */
    char* name;

    /** @brief Field value. */
    char* value;
}
Bibtex_field;


/** @brief BIBTEX entry. */
typedef struct
{
    /** @brief Type of entry. */
    char* type;

    /** @brief Unique entry key. */
    char* key;

    /** @brief Array of BIBTEX fields. */
    Bibtex_field** fields;

    /** @brief Number of used fields in the entry. */
    uint32_t num_fields;

    /** @brief Number of unused fields in the entry. */
    uint32_t num_unused_fields;
}
Bibtex_entry;


/** @brief Creates a Bibtex_entry. */
void create_bibtex_entry
(
    Bibtex_entry** entry_out, 
    const char*    type, 
    const char*    key
);


/** @brief Frees a Bibtex_entry. */
void free_bibtex_entry(Bibtex_entry* entry);


/** @brief Adds a field to a Bibtex_entry. */
void add_field_to_bibtex_entry
(
    Bibtex_entry* entry,
    const char*   name,
    const char*   value
);


/** @brief Writes a Bibtex_entry to a file stream. */
void write_bibtex_entry(Bibtex_entry* entry, FILE* fp);


#endif
