%{
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#ifdef BIBTEXQ_HAVE_DMALLOC
#include <dmalloc.h>
#endif

#include "bibtex_entry.h"

extern int main(int, char**);
extern void query_bibtex_entry(Bibtex_entry*);

int yyerror(char *);
int yylex();

/*
static void get_keywords(char** str_out, const char* str)
{
    int i, j, len;
    char c;

    assert(*str_out == NULL);

    len = strlen(str);

    if (len < 1) return;

    i = 0;
    while ((i < len) &&
       ((c = str[i]) == ' ' || c == '\t' || c == '\n'))
    {
        i++;
    }

    if (i == len) return;

    assert(*str_out = malloc((len+1)*sizeof(char)));

    j = 0;
    (*str_out)[j] = str[i++];
    
    for (; i < len; i++)
    {
        c = str[i];

        if ((*str_out)[j] == ',' && (c == ',' || c == '\t' || c == '\n'))
        {
            continue;
        }
        else if ((*str_out)[j] == ' ' && (c == ' ' || c == '\t' || c == '\n'))
        {
            continue;
        }
        else if ((*str_out)[j] != ' ' && (c == '\t' || c == '\n'))
        {
            (*str_out)[++j] = ' ';
        }
        else
        {
            (*str_out)[++j] = c;
        }
    }
}
*/

extern Bibtex_entry* entry;

%}


%union
{
    char *string;
    int  integer;
}

%token <string> ENTRY
%token <string> COMMENT
%token <string> IGNORE

%token <string> ENTRY_TYPE
%token <string> BEGIN_ENTRY
%token <string> END_ENTRY

%token <string> ENTRY_KEY
%token <string> FIELD_SEPARATOR
%token <string> EMPTY_ENTRY

%token <string> KEYWORDS_FIELD
%token <string> GENERIC_FIELD

%token <string> BEGIN_FIELD_VALUE

%token <string> FIELD_VALUE
%token <string> LEFT_BRACE
%token <string> LEFT_BRACE_KEEP
%token <string> RIGHT_BRACE
%token <string> RIGHT_BRACE_KEEP



%%

entries : entry entries
        | COMMENT entries { $<string>$ = $1; }
        |
        ;

entry : new_entry FIELD_SEPARATOR fields END_ENTRY
        { query_bibtex_entry(entry); }
      | new_entry EMPTY_ENTRY END_ENTRY 
        { query_bibtex_entry(entry); }
      | error {write_bibtex_entry(entry, stderr); fprintf(stderr, "Syntax error in bibtex file\n"); return -1;}
      ;

new_entry : ENTRY ENTRY_TYPE BEGIN_ENTRY ENTRY_KEY 
            { create_bibtex_entry(&entry, $2, $4); free($2); free($4); }
          | error {write_bibtex_entry(entry, stderr); fprintf(stderr, "Syntax error in bibtex file\n"); return -1;}
          ;

fields : field FIELD_SEPARATOR fields
       | field
       ;

field : GENERIC_FIELD BEGIN_FIELD_VALUE field_value
         {
             add_field_to_bibtex_entry(entry, $1, $<string>3);
             free($1);
             free($<string>3);
         }
       | KEYWORDS_FIELD BEGIN_FIELD_VALUE field_value
         {
             /* Could customize this. */
             add_field_to_bibtex_entry(entry, $1, $<string>3);
             free($1);
             free($<string>3);
         }
       | error {write_bibtex_entry(entry, stderr); fprintf(stderr, "Syntax error in bibtex file\n"); return -1;}
       ;

field_value : FIELD_VALUE field_value
              {
                  int len_1;
                  int len_2;
                  int len;
                  char* str_1 = $1;
                  char* str_2 = $<string>2;
                  char* str;

                  if (str_2 != NULL) 
                  {
                      len_1 = strlen(str_1);
                      len_2 = strlen(str_2);
                      len   = len_1 + len_2;

                      assert(str = malloc((len+1)*sizeof(char)));
                      strncpy(str, str_1, len_1);
                      strncpy((str+len_1), str_2, len_2);
                      str[len] = '\0';

                      $<string>$ = str;

                      free($1);
                      free($<string>2);
                  }
                  else
                  {
                      $<string>$ = str_1;
                  }
              }
            | LEFT_BRACE field_value RIGHT_BRACE field_value
              {
                  int len_1;
                  int len_2;
                  int len;
                  char* str_1 = $<string>2;
                  char* str_2 = $<string>4;
                  char* str;

                  if (str_1 == NULL && str_2 == NULL)
                  {
                      $<string>$ = NULL;
                  }
                  else if (str_1 == NULL)
                  {
                      $<string>$ = str_2;
                  }
                  else if (str_2 == NULL)
                  {
                      $<string>$ = str_1;
                  }
                  else
                  {
                      len_1 = strlen(str_1);
                      len_2 = strlen(str_2);
                      len   = len_1 + len_2;

                      assert((str = malloc((len+1)*sizeof(char))) != NULL);
                      strncpy(str, str_1, len_1);
                      strncpy((str+len_1), str_2, len_2);
                      str[len] = '\0';

                      $<string>$ = str;
                      free($<string>2);
                      free($<string>4);
                  }

                  free($1);
                  free($3);
              }
            | LEFT_BRACE_KEEP field_value RIGHT_BRACE_KEEP field_value
              {
                  int len_1;
                  int len_2;
                  int len;
                  char* str_1 = $<string>2;
                  char* str_2 = $<string>4;
                  char* str;
                  char* left = $1;
                  char* right = $3;

                  if (str_1 == NULL && str_2 == NULL)
                  {
                      $<string>$ = NULL;
                  }
                  else if (str_1 == NULL)
                  {
                      $<string>$ = str_2;
                  }
                  else if (str_2 == NULL)
                  {
                      len_1 = strlen(str_1);
                     
                      assert(str = malloc((len_1+3)*sizeof(char)));
                      str[0] = left[0];
                      strncpy(str+1, str_1, len_1);
                      str[len_1+1] = right[0];
                      str[len_1+2] = '\0';

                      $<string>$ = str;
                      free($<string>2);
                      free($<string>4);
                  }
                  else
                  {
                      len_1 = strlen(str_1);
                      len_2 = strlen(str_2);
                      len   = len_1 + len_2 + 2;

                      assert(str = malloc((len+1)*sizeof(char)));
                      str[0] = left[0];
                      strncpy(str+1, str_1, len_1);
                      str[len_1+1] = right[0];
                      strncpy((str+len_1+2), str_2, len_2);
                      str[len] = '\0';

                      $<string>$ = str;
                      free($<string>2);
                      free($<string>4);
                  }

                  free($1);
                  free($3);
              }
            | { $<string>$ = NULL; }
            | error {write_bibtex_entry(entry, stderr); fprintf(stderr, "Syntax error in bibtex file\n"); return -1;}
            ;

%%

int yyerror( char *s )
{
    // ignore this error message, we have our own
        return -1;
}

int yywrap( )
{
    return 1;
}
