%Start COMMENT_ENTRY_STATE NESTED_COMMENT_ENTRY_STATE ENTRY_STATE ENTRY_KEY_STATE FIELD_NAME_STATE FIELD_VALUE_STATE FIELD_VALUE_SENTENCE_STATE FIELD_VALUE_WORD_STATE 

%{
#include "grammar.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef BIBTEXQ_HAVE_DMALLOC
#include <dmalloc.h>
#endif

YYSTYPE yylval;

#undef yywrap
#define YY_NO_UNPUT

static int nested_level = 0;

static void strip(char* str_dest, const char* str_src, int len, const char* strip_chars, int n)
{
    int i, j, k;
    int strip_it;
    char c, d;

    k = 0;
    for (i = 0; i < len; i++)
    {
        c = str_src[i];
        strip_it = 0;

        for (j = 0; j < n; j++)
        {
            d = strip_chars[j];
            if (c == d)
            {
                strip_it = 1;
                break;
            }
        }

        if (!strip_it) str_dest[k++] = str_src[i];
    }

    str_dest[k] = '\0';
}


%}

alpha           [A-Za-z]
number          [0-9]
connector       [-_:.~]
others          [%#@$,*/?='();<>&+!"]
backspecial     [\\]([~#&|;<>/"']|{alpha}|{number})
word            ({alpha}|{number}|{connector})+
spaces          [ \t\n]
sentence        ({word}|{others}|{backspecial}|{spaces})+



%%

<INITIAL>[@]comment{spaces}*[{]{spaces}* {
        BEGIN(COMMENT_ENTRY_STATE);
        return COMMENT;
        }

<INITIAL>[@]{spaces}* {
        BEGIN(ENTRY_STATE);
        return ENTRY;
        }

<INITIAL>[%].*[\n] {
        return COMMENT;
        }





<COMMENT_ENTRY_STATE>[{]{spaces}* {
        nested_level++;
        BEGIN(NESTED_COMMENT_ENTRY_STATE);
        return COMMENT;
        }

<COMMENT_ENTRY_STATE>[}]{spaces}* {
        BEGIN(INITIAL);
        return COMMENT;
        }

<COMMENT_ENTRY_STATE>{sentence} {
        return COMMENT;
        }

<NESTED_COMMENT_ENTRY_STATE>[}]{spaces}* {
        nested_level--;
        if (nested_level == 0)
        {
            BEGIN(COMMENT_ENTRY_STATE);
        }
        return COMMENT;
        }

<NESTED_COMMENT_ENTRY_STATE>{sentence} {
        return COMMENT;
        }





<ENTRY_STATE>{word}{spaces}* {
        int len = strlen(yytext);
        yylval.string = malloc(sizeof(char)*(len+1));
        strip(yylval.string, yytext, len, " \t\n", 3);
        /*fprintf(stderr, "Entry: %s\n", yylval.string);*/
        return ENTRY_TYPE;
        }

<ENTRY_STATE>[{]{spaces}* {
        BEGIN(ENTRY_KEY_STATE);
        return BEGIN_ENTRY;
        }

<ENTRY_STATE>[}] {
        BEGIN(INITIAL);
        return END_ENTRY;
        }





<ENTRY_KEY_STATE>{word}{spaces}* {
        int len = strlen(yytext);
        yylval.string = malloc(sizeof(char)*(len+1));
        strip(yylval.string, yytext, len, ", \t\n", 4);
        /*fprintf(stderr, "Key: %s\n", yylval.string);*/
        return ENTRY_KEY;
        }

<ENTRY_KEY_STATE>[,] {
        BEGIN(FIELD_NAME_STATE);
        return FIELD_SEPARATOR;
        }

<ENTRY_KEY_STATE>{spaces}* {
        BEGIN(ENTRY_STATE);
        return EMPTY_ENTRY;
        }




<FIELD_NAME_STATE>keywords{spaces}* {
        int len = strlen(yytext);
        yylval.string = malloc(sizeof(char)*(len+1));
        strip(yylval.string, yytext, len, " \t\n", 3);
        /*fprintf(stderr, "%s = ", yylval.string);*/
        BEGIN(FIELD_VALUE_STATE);
        return KEYWORDS_FIELD;
        }

<FIELD_NAME_STATE>{word}{spaces}* {
        int len = strlen(yytext);
        yylval.string = malloc(sizeof(char)*(len+1));
        strip(yylval.string, yytext, len, " \t\n", 3);
        /*fprintf(stderr, "%s = ", yylval.string);*/
        BEGIN(FIELD_VALUE_STATE);
        return GENERIC_FIELD;
        }





<FIELD_VALUE_STATE>{spaces}*[=]{spaces}* {
        BEGIN(FIELD_VALUE_SENTENCE_STATE);
        return BEGIN_FIELD_VALUE;
        }

<FIELD_VALUE_STATE>{spaces}*[,]{spaces}* {
        BEGIN(FIELD_NAME_STATE);
        return FIELD_SEPARATOR;
        }

<FIELD_VALUE_STATE>{spaces}*[}]{spaces}* {
        BEGIN(INITIAL);
        return END_ENTRY;
        }



<FIELD_VALUE_SENTENCE_STATE>{sentence} {
        int len = strlen(yytext);
        yylval.string = malloc(sizeof(char)*(len+1));
        strncpy(yylval.string, yytext, len);
        yylval.string[len] = '\0';
        return FIELD_VALUE;
        }

<FIELD_VALUE_SENTENCE_STATE>[{"] {
        yylval.string = malloc(sizeof(char)*2);
        yylval.string[0] = yytext[0];
        yylval.string[1] = '\0';
        nested_level++;
        if (nested_level == 1)
        {
           return LEFT_BRACE;
        }
        
        return LEFT_BRACE_KEEP;
        }

<FIELD_VALUE_SENTENCE_STATE>[}"] {
        yylval.string = malloc(sizeof(char)*2);
        yylval.string[0] = yytext[0];
        yylval.string[1] = '\0';
        --nested_level;
        if (nested_level == 0)
        {
            BEGIN(FIELD_VALUE_STATE);
            return RIGHT_BRACE;
        }
        return RIGHT_BRACE_KEEP;
        }






<FIELD_VALUE_WORD_STATE>{word} {
        int len = strlen(yytext);
        yylval.string = malloc(sizeof(char)*(len+1));
        strncpy(yylval.string, yytext, len);
        yylval.string[len] = '\0';
        BEGIN(FIELD_VALUE_STATE);
        return FIELD_VALUE;
        }




{spaces}* { ; }
